import logging

import boto3

from contextlib import closing

from django.conf import settings
from django.core.files.temp import gettempdir, NamedTemporaryFile

client = boto3.client('polly')
logger = logging.getLogger(__name__)


def synthesize_speech_from_page(request, page):
    """
    Synthesize speech from a specific Wagtail page.
    """
    if hasattr(page, 'get_speech_text'):
        logger.info("Synthesize speech for %s" % page)
        text = page.get_speech_text(request)
        if not text:
            logger.warning("No speech text found for %s" % page)
            return False
        text_type = 'ssml' if text.startswith('<speak>') else 'text'
        output_format = getattr(
            settings, 'WAGTAIL_SPEECH_OUTPUT_FORMAT', 'mp3')
        response = client.synthesize_speech(
            OutputFormat=output_format,
            SampleRate=getattr(settings, 'WAGTAIL_SPEECH_SAMPLE_RATE', '8000'),
            Text=text,
            TextType=text_type,
            VoiceId=getattr(settings, 'WAGTAIL_SPEECH_VOICE_ID', 'Joey'),
        )
        if 'AudioStream' in response and hasattr(page, 'audio_stream'):
            with closing(response['AudioStream']) as stream:
                try:
                    temp_dir = gettempdir()
                    temp_file = NamedTemporaryFile(dir=temp_dir)
                    temp_file.write(stream.read())
                    page.audio_stream.save("%s.%s" % (
                        page.slug, output_format[:3]), temp_file)
                except IOError as error:
                    logger.error("An error occured: %s" % error)


def get_speech_text_from_stream_field(request, field, context=None):
    """
    Get speech text from a StreamField.

    This methods iterates over the stream field blocks which have the method
    get_speech_text implemented. It concats these values into a compatible
    multiline string value.
    """
    values = []
    for child in field:
        if hasattr(child.block, 'get_speech_text'):
            values.append(child.block.get_speech_text(child.value))
    text = '<break strength=\"x-strong\"/>'.join(values)
    return "<speak>%s</speak>" % text



from wagtail.wagtailcore import hooks

from wagtailspeech.utils import synthesize_speech_from_page


@hooks.register('after_edit_page')
def synthesize_speech(request, page):
    synthesize_speech_from_page(request, page)



from setuptools import find_packages, setup


install_requires = [
    'boto3>=1.4.4',
    'django>=1.8',
    'wagtail>=1.2'
]

test_require = [
    'flake8',
    'isort',
    'pytest',
    'pytest-cov',
    'pytest-django',
    'wagtail',
]

setup(
    name='wagtail-speech',
    version='0.1.0',
    description='Turn Wagtail pages into lifelike speech using Amazon Polly.',
    author='Rob Moorman',
    author_email='rob@moori.nl',
    url='https://github.com/moorinteractive/wagtail-speech',
    license='MIT',
    install_requires=install_requires,
    extras_require={
        'test': test_require,
    },
    package_dir={'': 'src'},
    packages=find_packages('src'),
    include_package_data=True,
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Operating System :: Unix',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5'
    ]
)



from django.db import models
from django.utils.translation import ugettext as _
from wagtail.wagtailcore.models import Page


class SynthesizeSpeechMixin(models.Model):
    """
    Mixin class to support synthesize speech functionalities.

    Example:
        class HomePage(SynthesizeSpeechMixin, Page):
            def get_speech_text(self):
                return self.title
    """
    audio_stream = models.FileField(_("audio stream"), blank=True, null=True)

    class Meta:
        abstract = True

    def get_speech_text(self):
        return None
